const chalk    = require('chalk');
const Reporter = require('../lib/reporter');


const specReporter = new Reporter({

  failureIndex: 0,

  start() {
    process.stdout.write('\n');
    process.stdout.write('\n');
  },

  finish() {
    process.stdout.write('\n');

    const passingTaskTime = this.tasksPassed.reduce((n, t) =>
      t.finishedAt.getTime() - t.startedAt.getTime() + n, 0);

    if (this.flowsPassed.length > 0) {
      process.stdout.write(
        `  ${chalk.green(`${this.flowsPassed.length} passing`)} ` +
        `${chalk.grey(`(${passingTaskTime}ms)`)}\n\n`
      );
    }
    if (this.flowsFailed.length > 0) {
      process.stdout.write(`  ${chalk.red(`${this.flowsFailed.length} failed`)}\n\n`);
    }

    if (this.isPass) {
      const s = (this.totalTime / 1000).toFixed(2);
      process.stdout.write(`✨  Done in ${s}s.\n`);
      return;
    }

    for (let i = 0; i < this.flowsFailed.length; i += 1) {
      const failedFlow      = this.flowsFailed[i];
      const firstFailedTask = failedFlow.tasksFailed[0];
      process.stdout.write(
        `${i + 1}) ${failedFlow.description}: ${firstFailedTask.description}\n`
      );
      process.stdout.write(`  ${chalk.red(firstFailedTask.error.stack)}\n\n`);
    }
    process.stdout.write(`\n${chalk.red('error')} Command failed with exit code 1.\n`);
  },

  finishFlow(f) {
    if (f.isPass) {
      process.stdout.write(`  ${chalk.green('✓')} `);
      process.stdout.write(chalk.grey(f.description));
      if (f.taskTime > 75) {
        process.stdout.write(` ${chalk.red(`(${f.taskTime}ms)`)}`);
      } else if (f.taskTime > 75 / 2) {
        process.stdout.write(` ${chalk.yellow(`(${f.taskTime}ms)`)}`);
      }
      process.stdout.write('\n');
      return;
    }

    process.stdout.write(`  ${f.description}\n`);

    for (let j = 0; j < f.tasksPassed.length; j += 1) {
      process.stdout.write(`    ${chalk.green('✓')} `);
      process.stdout.write(chalk.grey(f.tasksPassed[j].description));
      if (f.taskTime > 50) {
        process.stdout.write(chalk.grey(`(${f.tasksPassed[j].taskTime}ms)`));
      }
      process.stdout.write('\n');
    }

    const firstFailedTask = f.tasksFailed[0];
    process.stdout.write(
      `    ${chalk.red(`${this.failureIndex += 1}) ${firstFailedTask.description}`)} \n`
    );
  },
});


module.exports = specReporter;
